# Global parameters and constants
#
import taichi as ti
ti.init(arch=ti.gpu)
#
N  = 400        # window width & height
Nc = N/2        # on-screen coords of Origin {x=0,y=0}
Scale = 2.1/Nc  # scale, = -xmin and +xmax displayed
Rinf = 3        # radii of "infinite growth"
MAX_ITER = 50   # limit of iterations

# image buffers for Julia set and Mandelbrot set
#
JuliaPix  = ti.Vector.field(n=3, dtype=float, shape=(N,N))
MandelPix = ti.Vector.field(n=3, dtype=float, shape=(N,N))

#====================================================
#   complex_sqr()
# computes square of a complex number
#
@ti.func
#====================================================
def complex_sqr(z):
    return ti.Vector([z[0]**2-z[1]**2, z[1]*z[0]*2])

#====================================================
#   Julia()
# generates image of Julia set (RGB colors of its JuliaPix)
#
@ti.kernel
#====================================================
def Julia(t: float, cx: float, cy: float):
    c = ti.Vector([cx, cy])
    tmp = (t-0.5)**2
    for x,y in JuliaPix:
        z = ti.Vector([(x-Nc)*Scale, (y-Nc)*Scale])
        iter = 0
        while z.norm()<Rinf+t and iter<MAX_ITER:
            z = complex_sqr(z) + c
            iter += 1
## good colors:
##        w = z.norm()/Rinf -1
##        JuliaPix[x,y] = w, (1+ti.sin(w))*0.5, (1-ti.cos(w))*0.5  # good colors
        w = z.norm()/Rinf-1
        JuliaPix[x,y] = w, (1+ti.sin(w))*0.5, (1-ti.cos(w))*0.5  # good colors

#====================================================
#   Mandelbrot()
# generates image of Julia set (RGB colors of its JuliaPix)
#
@ti.kernel
#====================================================
def Mandelbrot(t: float):
    for x,y in MandelPix:
        c = ti.Vector([(x-Nc)*Scale, (y-Nc)*Scale])
        z = ti.Vector([0.0, 0.0])
        iter = 0
        while z.norm()<Rinf and iter<MAX_ITER:
            z = complex_sqr(z) + c
            iter += 1
        w = iter/MAX_ITER
        MandelPix[x,y] = 1-w, 1-w, 1-w

#====================================================
#   MAIN
#====================================================
from math import pi

# make app window
#
gui1 = ti.GUI("Julia Set", res=(N,N)) #, fast_gui=True )
#>>>try-new GGUI>>gui1 = ti.ui.Window("Julia Set", res=(N,N)) #, fast_gui=True )
gui2 = ti.GUI("Mandelbrot set", res=(N,N)) #, fast_gui=True)

### read path-points from file 
###
##F = open("points.txt")
##LS = F.readlines()
##F.close()
##
### convert text to the list of floats
###
##Path = []
##for s in LS:
##    x,y = map(float, s.strip().split())
##    Path.append( (x,y) )
##del LS

#prepare Julia set
#
Julia(0, 0.3, 0.5)
##if gui1.running:
##    gui1.set_image(JuliaPix)
##    gui1.show()

#prepare Mandelbrot set
#
Mandelbrot(0)
##if gui2.running:
##    gui2.set_image(MandelPix)
##    gui2.show()

while gui1.running and gui2.running:
    gui1.set_image(JuliaPix)
    gui1.show()
    gui2.set_image(MandelPix)
    gui2.show()

# iterate through path-points, making J_COUNT intermediate steps
#
##J_COUNT = 50
##for i in range(0,len(Path)-1):
##    if not gui1.running: break
##    x1 = Path[i][0]
##    y1 = Path[i][1]
##    dx = (Path[i+1][0]-x1)/J_COUNT
##    dy = (Path[i+1][1]-y1)/J_COUNT
##    for j in range(J_COUNT):
##        t = 2*pi*j/J_COUNT
##        cx = x1+dx*j
##        cy = y1+dy*j
##        Julia(t, cx, cy)
##
##        ix = int(cx/Scale+1.5*Nc)
##        iy = int(cy/Scale+Nc)
##        MandelPix[ix,iy] = 1.0, 0.5, 0.0
##        
##        if gui1.running:
##            gui1.set_image(JuliaPix)
##            gui1.show()
##        if gui2.running:
##            gui2.set_image(MandelPix)
##            gui2.show()
